package com.huaxu.mq;

import com.alibaba.fastjson.JSONObject;
import com.huaxu.Dto.MessageDto;
import com.huaxu.dao.MessageMapper;
import com.huaxu.dao.MessageTemplateMapper;
import com.huaxu.dao.MessageTypeMapper;
import com.huaxu.entity.Message;
import com.huaxu.entity.MessageTemplate;
import com.huaxu.entity.MessageType;
import com.huaxu.entity.WebsocketConnectInfo;
import com.huaxu.util.MessagePush;
import com.huaxu.util.Parser;
import com.huaxu.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
public class DispatchMessage {
    @Autowired
    private RedisUtil redisUtil;
    @Value("${user.connections}")
    private String userConnections;
    @Autowired
    private AmqpTemplate rabbitTemplate;
    @Autowired
    private MessageTypeMapper messageTypeMapper;
    @Autowired
    private MessageTemplateMapper messageTemplateMapper;
    @Autowired
    private MessageMapper messageMapper;
    @RabbitListener(queues = "${dispath.queue}")
    public void dispatch(String message){

        Long size = redisUtil.opsForList().size(userConnections);
        Message receiveMessage= JSONObject.parseObject(message, Message.class);
        receiveMessage.setStatus(1);
        receiveMessage.setMessageStatus(0);
        Integer messageTemplateId = receiveMessage.getMessageTemplateId();
        Integer messageTypeId = receiveMessage.getMessageType();
        MessageTemplate messageTemplate = messageTemplateMapper.selectById(messageTemplateId);
        MessageType messageType = messageTypeMapper.selectById(messageTypeId);
        MessageDto messageDto = new MessageDto();
        String receiveMessageContent=receiveMessage.getMessageContent();
        if(messageTemplate!=null&&receiveMessageContent.contains("{")){
            String content = messageTemplate.getContent();
            Map<String,String> param = JSONObject.parseObject(receiveMessageContent, Map.class);
            String replaceContent = Parser.replaceAllClearBlanks(content, param);
            receiveMessage.setMessageContent(replaceContent);
        }
        Integer channel=0;
        if(receiveMessage.getChannel()!=null){
            channel=receiveMessage.getChannel();
        }
        messageMapper.insert(receiveMessage);
        messageDto.setContent(receiveMessage.getMessageContent());
        messageDto.setType(messageType==null?"未知":messageType.getShortName());
        messageDto.setUserId(receiveMessage.getUserId()+"");
        messageDto.setTypeId(messageTypeId);
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String date=dtf2.format(now);
        messageDto.setDate(date);
        List<String> connectInfos = redisUtil.opsForList().range(userConnections, 0, size);
        if(channel==1){
            List<String>alias=new ArrayList<>();
            alias.add(receiveMessage.getUserId()+"");
            MessagePush.sendAlias(receiveMessage.getMessageContent(),alias);
            //app推送
        }else{
            connectInfos.forEach(connectInfo ->{
                WebsocketConnectInfo websocketConnectInfo =
                        JSONObject.parseObject(connectInfo, WebsocketConnectInfo.class);
                String sendMessage = JSONObject.toJSONString(messageDto);
                String connectUserId=websocketConnectInfo.getUserId();
                String receiveUserId=receiveMessage.getUserId()+"";
                if(connectUserId.equals(receiveUserId)){
                    log.info("向用户："+websocketConnectInfo.getUserId()+"发送消息："+sendMessage);

                    rabbitTemplate.convertAndSend(websocketConnectInfo.getConnectServerExchange(),
                            websocketConnectInfo.getConnectServerRoutingKey(), sendMessage);

                }
            });
        }

    }
}
